use crate::models::Claims;
use actix_web::body::{BoxBody, MessageBody};
use actix_web::dev::{forward_ready, Service, ServiceRequest, ServiceResponse, Transform};
use actix_web::{Error, HttpResponse};
use futures::future::{ok, Ready};
use serde_json::json;
use std::future::Future;
use std::pin::Pin;

pub struct AuthMiddleware;

impl<S, B> Transform<S, ServiceRequest> for AuthMiddleware
where
    S: Service<ServiceRequest, Response = ServiceResponse<B>, Error = Error>,
    S::Future: 'static,
    B: MessageBody + 'static,
{
    type Response = ServiceResponse<BoxBody>; // Changed to BoxBody
    type Error = Error;
    type Transform = AuthMiddlewareService<S>;
    type InitError = ();
    type Future = Ready<Result<Self::Transform, Self::InitError>>;

    fn new_transform(&self, service: S) -> Self::Future {
        ok(AuthMiddlewareService { service })
    }
}

pub struct AuthMiddlewareService<S> {
    service: S,
}

impl<S, B> Service<ServiceRequest> for AuthMiddlewareService<S>
where
    S: Service<ServiceRequest, Response = ServiceResponse<B>, Error = Error>,
    S::Future: 'static,
    B: MessageBody + 'static,
{
    type Response = ServiceResponse<BoxBody>; // Changed to BoxBody
    type Error = Error;
    type Future = Pin<Box<dyn Future<Output = Result<Self::Response, Self::Error>>>>;

    forward_ready!(service);

    fn call(&self, req: ServiceRequest) -> Self::Future {
        if req.path() == "/admin/login" || req.path() == "/admin/create" {
            let fut = self.service.call(req);
            return Box::pin(async move {
                let res = fut.await?;
                Ok(res.map_into_boxed_body()) // Convert the response body to BoxBody
            });
        }

        let auth_header = req.headers().get("Authorization");
        match auth_header {
            Some(header) => {
                let token = header.to_str().unwrap_or("").replace("Bearer ", "");
                if verify_token(&token) {
                    let fut = self.service.call(req);
                    Box::pin(async move {
                        let res = fut.await?;
                        Ok(res.map_into_boxed_body()) // Convert the response body to BoxBody
                    })
                } else {
                    let (request, _) = req.into_parts();
                    let response = HttpResponse::Unauthorized()
                        .json(json!({"error": "Invalid token"}))
                        .map_into_boxed_body();
                    Box::pin(async move { Ok(ServiceResponse::new(request, response)) })
                }
            }
            None => {
                let (request, _) = req.into_parts();
                let response = HttpResponse::Unauthorized()
                    .json(json!({"error": "No authorization token"}))
                    .map_into_boxed_body();
                Box::pin(async move { Ok(ServiceResponse::new(request, response)) })
            }
        }
    }
}

pub fn verify_token(token: &str) -> bool {
    let validation = jsonwebtoken::Validation::default();
    let key = jsonwebtoken::DecodingKey::from_secret("your-secret-key".as_ref());
    jsonwebtoken::decode::<Claims>(token, &key, &validation).is_ok()
}
