import type { Form, Submission, LoginCredentials } from './types';

const API_BASE_URL = 'http://127.0.0.1:8080';

/**
 * Helper to make authenticated requests.
 * @param endpoint The API endpoint (relative to base URL).
 * @param options Fetch options such as method, headers, and body.
 * @returns The JSON-parsed response.
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
async function authenticatedRequest(endpoint: string, options: RequestInit): Promise<any> {
	const token = localStorage.getItem('authToken'); // Replace with a secure token storage solution if needed
	if (!token) {
		throw new Error('Authentication token is missing. Please log in.');
	}

	const response = await fetch(`${API_BASE_URL}${endpoint}`, {
		...options,
		headers: {
			...options.headers,
			Authorization: `Bearer ${token}`, // Include the token in the Authorization header
			'Content-Type': 'application/json'
		}
	});

	if (!response.ok) {
		throw new Error(`Error: ${response.statusText}`);
	}

	return response.json();
}

/**
 * Create a new form (authenticated).
 * @param name The name of the form.
 * @param fields The fields of the form in JSON format.
 * @returns The ID of the created form.
 */
export async function createForm(name: string, fields: unknown): Promise<string> {
	return await authenticatedRequest('/forms', {
		method: 'POST',
		body: JSON.stringify({ name, fields })
	});
}

/**
 * Get all forms (authenticated).
 * @returns An array of forms.
 */
export async function getForms(): Promise<Form[]> {
	return await authenticatedRequest('/forms', { method: 'GET' });
}

/**
 * Get all submissions for a specific form (authenticated).
 * @param formId The ID of the form.
 * @returns An array of submissions for the form.
 */
export async function getSubmissions(formId: string): Promise<Submission[]> {
	return await authenticatedRequest(`/forms/${formId}/submissions`, { method: 'GET' });
}

/**
 * Submit a form (unauthenticated).
 * @param formId The ID of the form to submit.
 * @param data The submission data in JSON format.
 * @returns The ID of the created submission.
 */
export async function submitForm(formId: string, data: unknown): Promise<string> {
	const response = await fetch(`${API_BASE_URL}/forms/${formId}/submissions`, {
		method: 'POST',
		headers: {
			'Content-Type': 'application/json'
		},
		body: JSON.stringify(data)
	});

	if (!response.ok) {
		throw new Error(`Error submitting form: ${response.statusText}`);
	}

	return await response.json();
}

/**
 * Admin login to get a token.
 * @param credentials The login credentials (username and password).
 * @returns The generated JWT token if successful.
 */
export async function adminLogin(credentials: LoginCredentials): Promise<string> {
	const response = await fetch(`${API_BASE_URL}/admin/login`, {
		method: 'POST',
		headers: {
			'Content-Type': 'application/json'
		},
		body: JSON.stringify(credentials)
	});

	if (!response.ok) {
		throw new Error(`Error during admin login: ${response.statusText}`);
	}

	const data = await response.json();
	localStorage.setItem('authToken', data.token); // Store token locally
	return data.token;
}

/**
 * Create a new admin user.
 * @param user The login credentials for the admin user.
 * @returns A success message upon creation.
 */
export async function createAdmin(user: LoginCredentials): Promise<string> {
	const response = await fetch(`${API_BASE_URL}/admin/create`, {
		method: 'POST',
		headers: {
			'Content-Type': 'application/json'
		},
		body: JSON.stringify(user)
	});

	if (!response.ok) {
		throw new Error(`Error creating admin user: ${response.statusText}`);
	}

	const data = await response.json();
	return data.message;
}
